import json
import os
from typing import Optional, Tuple


def get_config_path(filename: str) -> map:
    paths = ["/etc/"]
    if os.geteuid():
        xdg_config_home = os.getenv("XDG_CONFIG_HOME")
        if xdg_config_home:
            paths.append(xdg_config_home)
        else:
            paths.append(os.path.expanduser("~/.config"))
    return map(lambda path: os.path.join(path, filename), paths)


def read_config() -> Optional[Tuple[str, str]]:
    paths = get_config_path("bit-user.json")
    for path in paths:
        try:
            with open(path) as f:
                data = json.loads(f.read())
                return data["username"], data["password"]
        except Exception:
            continue
    return None



import math
from base64 import b64encode
from html.parser import HTMLParser
from typing import Optional, Tuple
from urllib.parse import parse_qs, urlparse

import requests

API_BASE = "http://10.0.0.55"


def parse_homepage() -> Tuple[str, str]:
    """Parse homepage of 10.0.0.55 and get the acid + ip of current session

    Raises:
        Exception: Throw exception if acid not present in the redirected URL
        Exception: Throw exception if response text does not contain IP

    Returns:
        Tuple[str, str]: Both the ip and the acid of the current session
    """

    res = requests.get(API_BASE)

    # ac_id appears in the url query parameter of the redirected URL
    query = parse_qs(urlparse(res.url).query)
    ac_id = query.get("ac_id")

    if not ac_id:
        raise Exception("failed to get acid")

    # ip appears in the response HTML
    class IPParser(HTMLParser):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.ip = None

        def handle_starttag(self, tag, attrs):
            if tag == "input":
                attr_dict = dict(attrs)
                if attr_dict.get("name") == "user_ip":
                    self.ip = attr_dict["value"]

        def feed(self, *args, **kwargs):
            super().feed(*args, **kwargs)
            return self.ip

    parser = IPParser()
    ip = parser.feed(res.text)

    if not ip:
        raise Exception("failed to get ip")

    return ip, ac_id[0]


def get_user_info() -> Tuple[bool, Optional[str]]:
    """Get current logged in user info if exists

    Returns:
        tuple[bool, Optional[str]]
        - a boolean indicating whether the current IP is logged in
        - the username of the current logged in user if exists
    """

    is_logged_in = True
    username = None

    resp = requests.get(API_BASE + "/cgi-bin/rad_user_info")
    data = resp.text

    if data == "not_online_error":
        is_logged_in = False
    else:
        username = data.split(",")[0]

    return is_logged_in, username


def fkbase64(raw_s: str) -> str:
    """Encode string with a magic base64 mask"""
    trans = str.maketrans(
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
        "LVoJPiCN2R8G90yg+hmFHuacZ1OWMnrsSTXkYpUq/3dlbfKwv6xztjI7DeBE45QA",
    )
    ret = b64encode(bytes(ord(i) & 0xFF for i in raw_s))
    return ret.decode().translate(trans)


def xencode(msg, key) -> str:
    def sencode(msg, key):
        def ordat(msg, idx):
            if len(msg) > idx:
                return ord(msg[idx])
            return 0

        msg_len = len(msg)
        pwd = []
        for i in range(0, msg_len, 4):
            pwd.append(
                ordat(msg, i)
                | ordat(msg, i + 1) << 8
                | ordat(msg, i + 2) << 16
                | ordat(msg, i + 3) << 24
            )
        if key:
            pwd.append(msg_len)
        return pwd

    def lencode(msg, key) -> str:
        msg_len = len(msg)
        ll = (msg_len - 1) << 2
        if key:
            m = msg[msg_len - 1]
            if m < ll - 3 or m > ll:
                return ""
            ll = m
        for i in range(0, msg_len):
            msg[i] = (
                chr(msg[i] & 0xFF)
                + chr(msg[i] >> 8 & 0xFF)
                + chr(msg[i] >> 16 & 0xFF)
                + chr(msg[i] >> 24 & 0xFF)
            )
        if key:
            return "".join(msg)[0:ll]
        return "".join(msg)

    if msg == "":
        return ""
    pwd = sencode(msg, True)
    pwdk = sencode(key, False)
    if len(pwdk) < 4:
        pwdk = pwdk + [0] * (4 - len(pwdk))
    n = len(pwd) - 1
    z = pwd[n]
    y = pwd[0]
    c = 0x86014019 | 0x183639A0
    m = 0
    e = 0
    p = 0
    q = math.floor(6 + 52 / (n + 1))
    d = 0
    while 0 < q:
        d = d + c & (0x8CE0D9BF | 0x731F2640)
        e = d >> 2 & 3
        p = 0
        while p < n:
            y = pwd[p + 1]
            m = z >> 5 ^ y << 2
            m = m + ((y >> 3 ^ z << 4) ^ (d ^ y))
            m = m + (pwdk[(p & 3) ^ e] ^ z)
            pwd[p] = pwd[p] + m & (0xEFB8D130 | 0x10472ECF)
            z = pwd[p]
            p = p + 1
        y = pwd[0]
        m = z >> 5 ^ y << 2
        m = m + ((y >> 3 ^ z << 4) ^ (d ^ y))
        m = m + (pwdk[(p & 3) ^ e] ^ z)
        pwd[n] = pwd[n] + m & (0xBB390742 | 0x44C6F8BD)
        z = pwd[n]
        q = q - 1
    return lencode(pwd, False)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bitsrun']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['bitsrun = bitsrun.cli:main']}

setup_kwargs = {
    'name': 'bitsrun',
    'version': '3.1.1',
    'description': 'A headless login / logout script for 10.0.0.55',
    'long_description': 'None',
    'author': 'spencerwooo',
    'author_email': 'spencer.woo@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4',
}


setup(**setup_kwargs)



import argparse
import sys

from bitsrun.action import Action
from bitsrun.config import read_config
from bitsrun.user import User


def main():
    parser = argparse.ArgumentParser(description="Login to BIT network")
    parser.add_argument("action", choices=["login", "logout"], help="login or logout")
    parser.add_argument("-u", "--username")
    parser.add_argument("-p", "--password")
    parser.add_argument("-v", "--verbose", action="store_true")
    parser.add_argument("-s", "--silent", action="store_true")
    parser.add_argument("-nc", "--no-color", action="store_true")
    args = parser.parse_args()

    if args.username and args.password:
        user = User(args.username, args.password)
    elif conf := read_config():
        user = User(*conf)
    else:
        parser.print_usage()
        sys.exit(1)

    try:
        if args.action == "login":
            res = user.do_action(Action.LOGIN)

            # Output login result by default if not silent
            if not args.silent:
                print(f"{res.get('username')} ({res.get('online_ip')}) logged in")

        else:
            res = user.do_action(Action.LOGOUT)

            # Output logout result by default if not silent
            if not args.silent:
                print(res.get("online_ip"), "logged out")

        # Output direct result of response if verbose
        if args.verbose:
            if args.no_color:
                print("Info:", res)
            else:
                print("\33[34m[Info]\033[0m", res)

    except Exception as e:
        if args.no_color:
            print("Error:", e)
        else:
            print("\033[91m[Error]", e, "\033[0m")

        # Throw with error code 1 for scripts to pick up error state
        sys.exit(1)


if __name__ == "__main__":
    main()



import hmac
import json
from hashlib import sha1
from typing import Dict, Union

from requests import Session

from bitsrun.action import Action
from bitsrun.exception import (
    AlreadyLoggedOutException,
    AlreadyOnlineException,
    UsernameUnmatchedException,
)
from bitsrun.utils import fkbase64, get_user_info, parse_homepage, xencode

API_BASE = "http://10.0.0.55"
TYPE_CONST = 1
N_CONST = 200


class User:
    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password

        self.ip, self.acid = parse_homepage()
        self.session = Session()

    def do_action(self, action: Action) -> Dict[str, Union[str, int]]:
        # Check current state - whether device is logged in and whether current user the same as the provided one
        is_logged_in, username = get_user_info()

        if is_logged_in and action is Action.LOGIN:
            raise AlreadyOnlineException(f"{username}, you are already online")
        if not is_logged_in and action is Action.LOGOUT:
            raise AlreadyLoggedOutException("you have already logged out")

        # Raise exception only if username exists on this IP and command line arguments provided another username
        if username and username != self.username:
            raise UsernameUnmatchedException(
                f"current logged in user {username} and provided username {self.username} does not match"
            )

        # Perform login or logout action
        params = self._make_params(action)
        response = self.session.get(API_BASE + "/cgi-bin/srun_portal", params=params)
        return json.loads(response.text[6:-1])

    def _get_token(self) -> str:
        params = {"callback": "jsonp", "username": self.username, "ip": self.ip}
        response = self.session.get(API_BASE + "/cgi-bin/get_challenge", params=params)
        result = json.loads(response.text[6:-1])
        return result["challenge"]

    def _make_params(self, action: Action) -> Dict[str, str]:
        token = self._get_token()

        params = {
            "callback": "jsonp",
            "username": self.username,
            "action": action.value,
            "ac_id": self.acid,
            "ip": self.ip,
            "type": TYPE_CONST,
            "n": N_CONST,
        }

        data = {
            "username": self.username,
            "password": self.password,
            "acid": self.acid,
            "ip": self.ip,
            "enc_ver": "srun_bx1",
        }

        hmd5 = hmac.new(token.encode(), b"", "MD5").hexdigest()
        json_data = json.dumps(data, separators=(",", ":"))
        info = "{SRBX1}" + fkbase64(xencode(json_data, token))
        chksum = sha1(
            "{0}{1}{0}{2}{0}{3}{0}{4}{0}{5}{0}{6}{0}{7}".format(
                token,
                self.username,
                hmd5,
                self.acid,
                self.ip,
                N_CONST,
                TYPE_CONST,
                info,
            ).encode()
        ).hexdigest()

        params.update({"password": "{MD5}" + hmd5, "chksum": chksum, "info": info})
        return params
